-- Cruise Control Addon
--
-- Version 1.0
-- Autor Kryster
-- 
-- Change log:
-- 07/27/2017: Release 1.0


ccaddon = {};
ccaddon.modDir = g_currentModDirectory


function ccaddon:load()
	self.ccaddon = {};
	local vmax = self.cruiseControl.maxSpeed
	self.ccaddon.group1 = math.min(10,vmax)
	self.ccaddon.group2 = math.min(20,vmax)
	self.ccaddon.group3 = vmax
	self.ccaddon.activeGroup = 0
	self.ccaddon.spdChange = false
	self.ccaddon.lastSpd = 0
	self.ccaddon.showOverlay = true
	self.ccaddon.virtualAxis = 0.0
	self.ccaddon.axisType = 0
	self.ccaddon.axisChangeTime = 2000 -- Miliseconds
				
	self.ccaddon.overlays = {}		
	self.ccaddon.bgWidth  = 0.2
	self.ccaddon.bgHeight = 0.065

	self.ccaddon.overlays.speedBg = Overlay:new("hubBg1", Utils.getFilename("hudBg1.dds", ccaddon.modDir .. "resources/"), 0, 0, self.ccaddon.bgWidth, self.ccaddon.bgHeight)		
	self.ccaddon.overlays.barBg = Overlay:new("hudBg2", Utils.getFilename("hudBg2.dds", ccaddon.modDir .. "resources/"), 0, 0, self.ccaddon.bgWidth, self.ccaddon.bgHeight)
	self.ccaddon.overlays.bar = Overlay:new("hudBar", Utils.getFilename("hudBar.dds", ccaddon.modDir .. "resources/"), 0, 0, 0, self.ccaddon.bgHeight*46/64)
	
	self.ccaddon.fontSize = 0.015;
	
end;

function ccaddon:delete()
end;

function ccaddon:mouseEvent(posX, posY, isDown, isUp, button)
end;

function ccaddon:keyEvent(unicode, sym, modifier, isDown)
end;


function ccaddon:update(dt)
	if self.isEntered and self.isClient and not self.isHired then
		-- Check if CC has beed disengaged or changed externally
		if self.ccaddon.activeGroup > 0 then
			-- If CC is deactivated, select group 0
			if self.cruiseControl.state == Drivable.CRUISECONTROL_STATE_OFF and self.ccaddon.axisType == 0 then
				self.ccaddon.activeGroup = 0
			-- if changed, update active group
			elseif self.ccaddon.lastSpd ~= self.cruiseControl.speed then
				if self.ccaddon.virtualAxis==1 or self.ccaddon.axisType==0 then
					if self.ccaddon.activeGroup == 1 then
						self.ccaddon.group1 = self.cruiseControl.speed
						self.ccaddon.lastSpd = self.cruiseControl.speed
					elseif self.ccaddon.activeGroup == 2 then
						self.ccaddon.group2 = self.cruiseControl.speed
						self.ccaddon.lastSpd = self.cruiseControl.speed
					elseif self.ccaddon.activeGroup == 3 then
						self.ccaddon.group3 = self.cruiseControl.speed
						self.ccaddon.lastSpd = self.cruiseControl.speed
					end
				end
			end
		end
		
		
        -- do all the input handling
		if self:getIsActiveForInput(false) and self.cruiseControl.isActive then
			-- Toggle overlays
			if InputBinding.hasEvent(InputBinding.CCADDON_TOGGLEHUD) then
				self.ccaddon.showOverlay = not self.ccaddon.showOverlay
			end
			
			-- Toggle axis modes
			if InputBinding.hasEvent(InputBinding.CCADDON_AXISMODE_DIRECT) then
				if self.ccaddon.axisType ~= 1 then
					self.ccaddon.axisType = 1
				else
					self.ccaddon.axisType = 0
				end
			end
			
			if InputBinding.hasEvent(InputBinding.CCADDON_AXISMODE_JOYSTICK) then
				if self.ccaddon.axisType ~= 2 then
					self.ccaddon.axisType = 2
				else
					self.ccaddon.axisType = 0
				end
			end

			-- Save CC speeds
			-- Speed 1
			if InputBinding.hasEvent(InputBinding.CCADDON_SPD1SAVE) then
				self.ccaddon.group1 = self.cruiseControl.speed
			end
			
			-- Speed 2
			if InputBinding.hasEvent(InputBinding.CCADDON_SPD2SAVE) then
				self.ccaddon.group2 = self.cruiseControl.speed
			end
			
			-- Speed 3
			if InputBinding.hasEvent(InputBinding.CCADDON_SPD3SAVE) then
				self.ccaddon.group3 = self.cruiseControl.speed
			end
			
			-- Set CC speeds
			-- Speed 1 
			if InputBinding.hasEvent(InputBinding.CCADDON_SPD1SET) then
				if self.ccaddon.activeGroup ~= 1 then
					self.ccaddon.activeGroup = 1
					if self.ccaddon.axisType == 0 then
						self.ccaddon.lastSpd = self.ccaddon.group1
						self.ccaddon.spdChange = true
					end
				else
					self:setCruiseControlState(Drivable.CRUISECONTROL_STATE_OFF)
					self.ccaddon.activeGroup = 0
				end
			end
				
			-- Speed 2
			if InputBinding.hasEvent(InputBinding.CCADDON_SPD2SET) then
				if self.ccaddon.activeGroup ~= 2 then
					self.ccaddon.activeGroup = 2
					if self.ccaddon.axisType == 0 then
						self.ccaddon.lastSpd = self.ccaddon.group2
						self.ccaddon.spdChange = true
					end
				else
					self:setCruiseControlState(Drivable.CRUISECONTROL_STATE_OFF)
					self.ccaddon.activeGroup = 0
				end
			end
				
			-- Speed 3
			if InputBinding.hasEvent(InputBinding.CCADDON_SPD3SET) then
				if self.ccaddon.activeGroup ~= 3 then
					self.ccaddon.activeGroup = 3
					if self.ccaddon.axisType == 0 then
						self.ccaddon.lastSpd = self.ccaddon.group3
						self.ccaddon.spdChange = true
					end
				else
					self:setCruiseControlState(Drivable.CRUISECONTROL_STATE_OFF)
					self.ccaddon.activeGroup = 0
				end
			end
				
			-- Input axis
			if self.ccaddon.axisType>0 and self.ccaddon.activeGroup>0 then
				-- Use the axis
				local newSpd = 0
				if self.ccaddon.activeGroup == 1 then
					newSpd = self.ccaddon.group1
				elseif self.ccaddon.activeGroup == 2 then
					newSpd = self.ccaddon.group2
				elseif self.ccaddon.activeGroup == 3 then
					newSpd = self.ccaddon.group3
				end
				if self.ccaddon.axisType == 1 then
					-- Direct input type. Only positive values acceptable.
					local axisRead = math.abs(InputBinding.getAnalogInputAxis(InputBinding.CCADDON_AXIS_DIRECT))
					self.ccaddon.virtualAxis = 1.0
					if axisRead<0.9 then
						self.ccaddon.virtualAxis = axisRead/.9
						newSpd = math.floor(newSpd * self.ccaddon.virtualAxis *10 + 0.5)/10.0
						self.ccaddon.spdChange = (newSpd ~= self.ccaddon.lastSpd)
						self.ccaddon.lastSpd = newSpd
					else
						self.ccaddon.spdChange = (newSpd ~= self.ccaddon.lastSpd)
						self.ccaddon.lastSpd = newSpd
					end
					
				else  -- axisType must be 2
					-- Joystick input type. 
					local axisRead = InputBinding.getAnalogInputAxis(InputBinding.CCADDON_AXIS_JOYSTICK)
					if axisRead~=0 then
						self.ccaddon.virtualAxis = self.ccaddon.virtualAxis + dt/self.ccaddon.axisChangeTime*axisRead
						if self.ccaddon.virtualAxis>1 then
							self.ccaddon.virtualAxis = 1
						elseif self.ccaddon.virtualAxis<0 then
							self.ccaddon.virtualAxis = 0
						end
					end
					newSpd = math.floor(newSpd * self.ccaddon.virtualAxis * 10 + 0.5)/10.0
					self.ccaddon.spdChange = (newSpd ~= self.ccaddon.lastSpd)
					self.ccaddon.lastSpd = newSpd
				end
			else
				-- Don't use the axis
				self.ccaddon.virtualAxis = 0.0
			end
			
		end
		
		-- Set speed
		if self.ccaddon.spdChange then
			self:setCruiseControlMaxSpeed(self.ccaddon.lastSpd)
			if self.ccaddon.lastSpd>0 then
				self:setCruiseControlState(Drivable.CRUISECONTROL_STATE_ACTIVE)
			else
				self:setCruiseControlState(Drivable.CRUISECONTROL_STATE_OFF)
			end
			if g_server ~= nil then
				g_server:broadcastEvent(SetCruiseControlSpeedEvent:new(self, self.cruiseControl.speed), nil, nil, self)
			else
				g_client:getServerConnection():sendEvent(SetCruiseControlSpeedEvent:new(self, self.cruiseControl.speed))
			end;
			self.ccaddon.spdChange = false
		end

	end;
end;

function ccaddon:updateTick(dt)
end;

function ccaddon:readStream(streamId, connection)
end;

function ccaddon:writeStream(streamId, connection)
end;


function ccaddon:onEnter()
end;

function ccaddon:onLeave()
end;

function ccaddon:draw()
	-- Unly run code if player is controlling a vehicle
	if self.isEntered and not self.isHired and self.ccaddon.showOverlay then
		-- Correction for mile/km unit. Engine always use km so only display txt needs correction
		local factor = 1 + (g_inGameMenu.useMilesElement.state - 1) * 0.6
		-- Positions for overlay background
		self.ccaddon.bgPosX = g_currentMission.vehicleHudBg.x;
		self.ccaddon.bgPosY = g_currentMission.vehicleHudBg.y + g_currentMission.vehicleHudBg.height;
		self.ccaddon.bgWidth  = g_currentMission.vehicleHudBg.width

		-- Positions for overlay text
		self.ccaddon.posX = self.ccaddon.bgPosX + 0.006;
		self.ccaddon.posY = self.ccaddon.bgPosY + 0.044;

		-- Render the overlay background
		local shifty = 0
		local width = 0
		if self.ccaddon.axisType>0 then
			renderOverlay(self.ccaddon.overlays.barBg.overlayId, self.ccaddon.bgPosX, self.ccaddon.bgPosY, self.ccaddon.bgWidth, self.ccaddon.bgHeight)
			renderOverlay(self.ccaddon.overlays.bar.overlayId, self.ccaddon.bgPosX + self.ccaddon.bgWidth * 49/512, self.ccaddon.bgPosY + self.ccaddon.bgHeight * 9/64, self.ccaddon.bgWidth * 454/512 * self.ccaddon.virtualAxis, self.ccaddon.bgHeight * 46/64)
			shifty = self.ccaddon.bgHeight
		end
		renderOverlay(self.ccaddon.overlays.speedBg.overlayId, self.ccaddon.bgPosX, self.ccaddon.bgPosY + shifty, self.ccaddon.bgWidth, self.ccaddon.bgHeight)

		-- Output text
		local text = "CC Addon:"
		setTextColor(1,1,1,1)
		renderText(self.ccaddon.posX, self.ccaddon.posY + shifty, self.ccaddon.fontSize, text)
		width = getTextWidth(self.ccaddon.fontSize,text) + 0.008
		
		text = string.format("S1: %.1f", self.ccaddon.group1 / factor)
		if self.ccaddon.activeGroup == 1 then
			setTextColor(0.262,0.639,0.041,1)
		else
			setTextColor(1,1,1,1)
		end
		renderText(self.ccaddon.posX + width, self.ccaddon.posY + shifty, self.ccaddon.fontSize, text)
		width = width + getTextWidth(self.ccaddon.fontSize,text) + 0.008
		
		text = string.format("S2: %.1f", self.ccaddon.group2 / factor)
		if self.ccaddon.activeGroup == 2 then
			setTextColor(0.262,0.639,0.041,1)
		else
			setTextColor(1,1,1,1)
		end
		renderText(self.ccaddon.posX + width, self.ccaddon.posY + shifty, self.ccaddon.fontSize, text)
		width = width + getTextWidth(self.ccaddon.fontSize,text) + 0.008
		
		
		text = string.format("S3: %.1f", self.ccaddon.group3 / factor)
		if self.ccaddon.activeGroup == 3 then
			setTextColor(0.262,0.639,0.041,1)
		else
			setTextColor(1,1,1,1)
		end
		renderText(self.ccaddon.posX + width, self.ccaddon.posY + shifty, self.ccaddon.fontSize, text)

	end
end;
